#
# This file is part of ToolRack.
#
# ToolRack is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# ToolRack is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ToolRack.  If not, see <http://www.gnu.org/licenses/>.

'''Protocol class for collecting a process stdout/stderr.'''

from io import StringIO
from locale import getpreferredencoding
from asyncio import SubprocessProtocol


class ProcessParserProtocol(SubprocessProtocol):
    '''Collect process stdout and stderr.

    Line parser functions can be passed for stdout and stderr, and they are
    called on each full line of output.

    When the process ends, the ``future`` returns a tuple with the full stdout
    and stderr. Each tuple element is ``None`` if a parser is passed for that
    stream.

    Parameters:
        - future: an :class:`asyncio.Future` which is called with a tuple with
          (stdout, stderr) from the process once it it exits.
        - out_parser: an optional parser for the process standard output.
        - err_parser: an optional parser for the process standard error.

    '''

    def __init__(self, future, out_parser=None, err_parser=None):
        self.future = future
        self._outputs = {
            fd: StreamHelper(callback=parser)
            for fd, parser in enumerate((out_parser, err_parser), 1)}

    def pipe_data_received(self, fd, data):
        stream = self._outputs.get(fd)
        if not stream:
            return

        data = data.decode(getpreferredencoding(False))
        stream.receive_data(data)

    def connection_lost(self, exc):
        self._outputs[1].flush_partial()
        self._outputs[2].flush_partial()
        stdout = self._outputs[1].get_data()
        stderr = self._outputs[2].get_data()
        if exc:
            self.future.set_exception(exc)
        else:
            self.future.set_result((stdout, stderr))


class StreamHelper:
    '''Helper to cache data until full lines of text are received.

    This is useful to collect data from a stream and process them when full
    lines are received.
    For example::

      stream = StreamHelper(callback)
      stream.receive_data('line one\\nline two')
      stream.receive_data('continues here\\n')

    would call ``callback`` twice, one with ``'line one'`` and one with
    ``'line two continues here'``

    Parameters:
        - callback: an optional function which is called with full lines of
          text from the stream.
        - separator: the line separator

    '''

    def __init__(self, callback=None, separator='\n'):
        self.separator = separator
        self._callback = callback
        self._buffer = StringIO() if not callback else None
        self._partial = StringIO()

    def receive_data(self, data):
        '''Receive data and process them.

        If a ``callback`` has been passed to the class, it's called for each
        full line of text.

        '''
        if self._callback:
            self._parse_data(data)
        else:
            self._buffer.write(data)

    def get_data(self):
        '''Return the full content of the stream.'''
        if not self._buffer:
            return
        return self._buffer.getvalue() + self._partial.getvalue()

    def flush_partial(self):
        '''Flush and process pending data from a partial line.'''
        if not self._callback:
            return
        partial = self._partial.getvalue()
        if partial:
            self._callback(partial)

    def _parse_data(self, data):
        '''Process data parsing full lines.'''
        lines = data.split(self.separator)
        lines[0] = self._pop_partial() + lines[0]
        self._partial.write(lines.pop())
        # Call the callback on full lines
        for line in lines:
            if line:
                self._callback(line)

    def _pop_partial(self):
        '''Return the current partial line and reset it.'''
        line = self._partial.getvalue()
        self._partial.truncate()
        return line
